#include <QWidget>
#include <QLineEdit>
#include <QPushButton>
#include <QHBoxLayout>
#include <QApplication>
#include <iostream>
#include <qlabel.h>
#include <qtextedit.h>
#include <QMessageBox>
#include <QRegularExpressionValidator>
#include <algorithm>
#include <sstream>
#include <QDebug>

using namespace std;
#define NO_OF_CHARS 256
void badCharHeuristic(const string& str, int size,
                      int badchar[NO_OF_CHARS])
{
    int i;
    for (i = 0; i < NO_OF_CHARS; i++)
        badchar[i] = -1;
    for (i = 0; i < size; i++)
        badchar[(int)str[i]] = i;
}

class SortingApp : public QWidget {
public:
    SortingApp(QWidget *parent = nullptr) : QWidget(parent) {
        setupUI();
    }
private:
    QLineEdit *inputField1;
    QLineEdit *inputField2;
    QTextEdit *outputField;
    QPushButton *testButton;
    void setupUI() {
        QVBoxLayout *mainLayout = new QVBoxLayout(this);

        QRegularExpressionValidator *validator = new QRegularExpressionValidator(QRegularExpression("[\\x00-\\x7F]*"), this);
        QLabel *label = new QLabel("Маска", this);
        mainLayout->addWidget(label);
        inputField1 = new QLineEdit(this);
        inputField1->setValidator(validator);
        mainLayout->addWidget(inputField1);
        QLabel *label2 = new QLabel("Текст", this);
        mainLayout->addWidget(label2);
        inputField2 = new QLineEdit(this);
        inputField2->setValidator(validator);
        mainLayout->addWidget(inputField2);
        
        QPushButton *findButton = new QPushButton("Виконати індивідуальне завдання", this);
        connect(findButton, &QPushButton::clicked, this, &SortingApp::find);
        QHBoxLayout *buttons = new QHBoxLayout();
        buttons->addWidget(findButton);
        mainLayout->addLayout(buttons);

        outputField = new QTextEdit(this);
        outputField->setReadOnly(true);
        mainLayout->addWidget(outputField);

        setLayout(mainLayout);
        setWindowTitle("alko11");
        resize(500, 400);
    }
private slots:
    vector<size_t> myfind(const std::string &S, const std::string &P) {
          vector<size_t>outme;
          int m = P.size();
          int n = S.size();
          int badchar[NO_OF_CHARS];
          badCharHeuristic(P, m, badchar);
          int s = 0;
          while (s <= (n - m)) {
              int j = m - 1;
              while (j >= 0 && P[j] == S[s + j]){
                  j--;
                }
              if (j < 0) {
                  outme.push_back(s);
                  s += (s + m < n) ? m - badchar[S[s + m]] : 1;
              }
              else
                  s += max(1, j - badchar[S[s + j]]);
          }
          return outme;
    }
    void find() {
      outputField->clear();
  
      if (inputField1->text().isEmpty() || inputField2->text().isEmpty()) {
          QMessageBox::warning(this, "Помилка", "Заповніть всі поля");
          return;
      }
      std::vector<std::string> parts;
      std::stringstream ss(inputField1->text().toStdString());
      std::string part;
      while (std::getline(ss, part, '*')) {
          if (!part.empty()) {
              parts.push_back(part);
          }
      }
      std::vector<std::vector<size_t>> results;
      for (const auto &p : parts) {
          results.push_back(myfind(inputField2->text().toStdString(), p));
          if (results.back().empty()) {
              outputField->append("Співпадінь для \"" + QString::fromStdString(p) + "\" не знайдено.");
              return;
          }
      }
      size_t n = results.size();
      size_t totalCombinations = 1;
      for (size_t i = 0; i < n; ++i) {
          totalCombinations *= results[i].size();
      }
      std::vector<size_t> indices(n, 0);
      for (size_t i = 0; i < totalCombinations; ++i) {
          std::vector<int> combination;
          for (size_t j = 0; j < n; ++j) if (!j || results[j][indices[j]]>combination[j-1])combination.push_back(results[j][indices[j]]);
              if (combination.size() == n) {
                  QString resultStr;
                  resultStr += QString::fromStdString(inputField2->text().toStdString().substr(combination[0], parts[0].size()));
                  for (size_t i = 1; i < n; i++) {
                      resultStr += QString::fromStdString(inputField2->text().toStdString().substr(combination[i - 1] + parts[i - 1].size(),parts[i].size()+combination[i] - parts[i - 1].size()-combination[i-1]));
                  }
                  outputField->append(resultStr);
              }
  
          // Оновлюємо індекси для наступної комбінації
          size_t j = n - 1;
          while (j < n) {
              if (indices[j] < results[j].size() - 1) {
                  ++indices[j];
                  break;
              } else {
                  indices[j] = 0;
                  --j;
              }
          }
      }
    }
};

int main(int argc, char *argv[]) {
    QApplication app(argc, argv);
    SortingApp sortingApp;
    sortingApp.show();
    return app.exec();
}
